precision highp float;

fshin vec2 vTexCoord;

uniform sampler2D uSampler;
uniform sampler2D lcSampler;
uniform sampler2D toneMapMask;
#if GLES3
uniform highp sampler3D uRgbMap;
#else
uniform highp sampler2D uRgbMap;
#endif

uniform float exposure;
uniform float exposureGain;
//uniform vec3 undoCurve[8];
uniform vec3 renderCurve[8];

//***uniforms
%s

uniform float uGlobalAmtHighlights;
uniform float uGlobalAmtShadows;
uniform float uGlobalAmtClarity;

uniform float uToneMapMid;

uniform vec4 lineLumWeightScale;
uniform vec4 lineLumWeightOffset;

uniform float uCompressionScale;
uniform float uCompressionOffset;

uniform float uHighlightPinAmount;

uniform float uToneMapMaxAbsOffset;

#if GLES3
out vec4 fragColor;
#else
#define fragColor gl_FragColor
#endif

const float kNonLinearMapping_sqrt_a = 1.0 / 16.0;

const float kNonLinearMapping_a = kNonLinearMapping_sqrt_a * kNonLinearMapping_sqrt_a;

const float kNonLinearMapping_scale1 = 1.06445122140;	// 1 / (sqrt (1 + a) - sqrt (a))
const float kNonLinearMapping_scale2 = 0.93945122137;	// 1 / scale1

vec2 EndpointPinMask (vec2 x)
{
    x = 1.0 - x;
    
    x *= x;     // 2
    x *= x;     // 4
    x *= x;     // 8
    
    return 1.0 - x;
}

const vec3 kGrayWeights = vec3 (0.30, 0.59, 0.11);

#if GLES3
// Tone mapping only in ES 3
vec3 DoToneMap( vec3 rgb, vec2 pt )
{
    // const kMaxFlareLinear_real64 = 0.000244140625;
    // const flareLinear = kMaxFlareLinear_real64;
    // const flareLog = Math.log2( flareLinear );
    // const eps = flareLinear / 128.0;
    
    const float flareLog = -12.0;
    const float eps = 0.00000190734;
    
    vec2 absGlobalAmtHS = abs(vec2(uGlobalAmtHighlights,uGlobalAmtShadows));
    
    float mask = texture (toneMapMask, pt).x;
    float srcGrayLinear = dot (rgb, kGrayWeights);
    float srcGrayLog = log2 (srcGrayLinear);
    
    vec4 lineLumWeightResult = lineLumWeightScale * srcGrayLog + lineLumWeightOffset;
    
    float partSwitch = step(srcGrayLog,uToneMapMid);
    
    float baseCompressed = mix(lineLumWeightResult.y, lineLumWeightResult.x, partSwitch);
    
    float localContrastSignal = srcGrayLog + mask - baseCompressed;
    localContrastSignal *= max(uGlobalAmtClarity, 0.0);
    localContrastSignal *= clamp(uCompressionScale * 0.5*(-uGlobalAmtHighlights + uGlobalAmtShadows) + uCompressionOffset, 1.0, 2.0);
    
    vec2 localContrastSignal2 = vec2(max(localContrastSignal, 0.0),min(localContrastSignal, 0.0));
    
    vec2 lumWeight;
    lumWeight.x = mix(lineLumWeightResult.w, lineLumWeightResult.z, partSwitch);
    
    lumWeight.x = clamp(lumWeight.x,0.0,1.0);
    lumWeight.y = 1.0-lumWeight.x;
    
    vec2 claritySHPinMask = EndpointPinMask(lumWeight);
    
    vec2 hsPinMask;
    hsPinMask.y =  mix(0.5 + 0.5*max(1.0-sign(uGlobalAmtShadows),0.0), 1.0, claritySHPinMask.x);
    hsPinMask.x = mix (1.0, 0.5, (1.0-claritySHPinMask.y)*max(-sign(uGlobalAmtHighlights),0.0));
    hsPinMask.x = mix(1.0, hsPinMask.x, max(uHighlightPinAmount,0.0));
    
    float baseOffset = uToneMapMaxAbsOffset * (uGlobalAmtHighlights + uGlobalAmtShadows)/ max(max(absGlobalAmtHS.x, absGlobalAmtHS.y), eps);
    vec2 offsetHS = lumWeight * absGlobalAmtHS * baseOffset;
    vec2 deltaHS = vec2(-uGlobalAmtHighlights, uGlobalAmtShadows);
    deltaHS = clamp(deltaHS,-1.0,1.0);
    deltaHS *= vec2(min(mask, 0.0),max(mask, 0.0));
    deltaHS += offsetHS;
    
    float delta = dot(deltaHS,hsPinMask);
    delta += dot(localContrastSignal2,claritySHPinMask);
    
    float startpoint;
    float t;
    
    partSwitch = sign(delta);
    float zeroSwitch = 1.0 - abs(partSwitch);
    partSwitch = 1.0-max(partSwitch,0.0);
    
    float flare = partSwitch*flareLog;
    startpoint = flare - (delta + delta);
    
    float t1 = step(startpoint,srcGrayLog);
    float t2 = step(srcGrayLog,startpoint);
    
    t = clamp((srcGrayLog - startpoint) / (flare - startpoint + zeroSwitch),0.0,1.0);
    t *= t * (1.0-mix(t2,t1,partSwitch));
    delta = mix (delta, 0.0, t);
    
    rgb *= exp2 (srcGrayLog + delta) / max(srcGrayLinear, eps);
    
    return max (rgb, 0.0);
}
#endif

vec3 sRGBGamma(in vec3 col)
{
    vec3 sV = step(col,vec3(0.0031308));
    return mix((1.055 * pow (col, vec3(1.0/2.4)) - 0.055),(col*12.92),sV);
}

vec3 invSRGBGamma(in vec3 col)
{
    vec3 sV = step(col,vec3(0.040449936));
    return mix(pow ((col + 0.055)/1.055, vec3(2.4)),(col/12.92),sV);
}

float calcProtection(in float hue, in float sat)
{
    float val = mod(hue + 1.0,6.0);
    float minSide = min(val,1.0);
    float maxSide = clamp(1.0 - (val - 1.5) * 4.0,0.0,1.0);
    val = min(minSide, maxSide);
    return val * (1.0 - sat*sat);
}

vec3 rgb2HSL(in vec3 rgbCol)
{
    vec3 colFactors;
    vec3 outVal;
    
    outVal.y = min(min(rgbCol.r, rgbCol.g), rgbCol.b);
    outVal.z = max(max(rgbCol.r, rgbCol.g), rgbCol.b);
    
    float delta = outVal.z - outVal.y;
    
    float deltaFactor = sign(delta);
    
    delta += 1.0 - deltaFactor;
    
    colFactors.xyz = 1.0 - sign(outVal.z - rgbCol.rgb);
    colFactors.yzx *= 1.0 - colFactors.xyz;
    
    outVal.x = ((rgbCol.g - rgbCol.b) / delta)*colFactors.x;
    outVal.x += (2.0+(rgbCol.b - rgbCol.r) / delta)*colFactors.y;
    outVal.x += (4.0+(rgbCol.r - rgbCol.g) / delta)*colFactors.z;
    
    outVal.x *= deltaFactor;
    outVal.x = mod(outVal.x+6.0,6.0);
    
    return outVal;
}

vec3 hsl2RGB(in vec3 hslCol)
{
    float dH = floor(hslCol.x);
    float h = fract(hslCol.x);
    
    if(mod(dH,2.0) > 0.5)
        h = 1.0 - h;
    
    vec3 cCol;
    vec3 outCol;
    
    cCol.x = hslCol.y;
    cCol.y = hslCol.z;
    cCol.z = (cCol.y - cCol.x) * h + cCol.x;
    
    outCol = mix(cCol.yzx,cCol.zyx,float(dH == 1.0));
    outCol = mix(outCol,cCol.xyz, float(dH == 2.0));
    outCol = mix(outCol,cCol.xzy, float(dH == 3.0));
    outCol = mix(outCol,cCol.zxy, float(dH == 4.0));
    outCol = mix(outCol,cCol.yxz, float(dH == 5.0));
    
    return outCol;
}
float spline6Value(in vec3 curve[6], in float t)
{
    float checkT;
    vec3 tVec;
    float tMask;
    float outVal;
    
    t *= 6.0;
    checkT = t;
    t = fract(t);
    tVec.x = 1.0;
    tVec.y = t;
    tVec.z = t*t;
    
    tMask = step(checkT,1.0); outVal = dot(curve[0],tVec)*tMask;
    tMask = step(checkT,2.0)*step(1.0,checkT); outVal += dot(curve[1],tVec)*tMask;
    tMask = step(checkT,3.0)*step(2.0,checkT); outVal += dot(curve[2],tVec)*tMask;
    tMask = step(checkT,4.0)*step(3.0,checkT); outVal += dot(curve[3],tVec)*tMask;
    tMask = step(checkT,5.0)*step(4.0,checkT); outVal += dot(curve[4],tVec)*tMask;
    tMask = step(5.0,checkT); outVal += dot(curve[5],tVec)*tMask;
    
    return outVal;
}

float spline8Value(in vec3 curve[8], in float t)
{
    float checkT; vec3 tVec; float tMask; float outVal;
    
    t *= 8.0;
    checkT = t;
    t = fract(t);
    tVec.x = 1.0;
    tVec.y = t;
    tVec.z = t*t;
    
    tMask = step(checkT,1.0);
    outVal = dot(curve[0],tVec)*tMask;
    
    tMask = step(checkT,2.0)*step(1.0,checkT);  outVal += dot(curve[1],tVec)*tMask;
    tMask = step(checkT,3.0)*step(2.0,checkT);  outVal += dot(curve[2],tVec)*tMask;
    tMask = step(checkT,4.0)*step(3.0,checkT);  outVal += dot(curve[3],tVec)*tMask;
    tMask = step(checkT,5.0)*step(4.0,checkT);  outVal += dot(curve[4],tVec)*tMask;
    tMask = step(checkT,6.0)*step(5.0,checkT);  outVal += dot(curve[5],tVec)*tMask;
    tMask = step(checkT,7.0)*step(6.0,checkT);  outVal += dot(curve[6],tVec)*tMask;
    tMask = step(7.0,checkT);                   outVal += dot(curve[7],tVec)*tMask;
    
    return outVal;
}

vec3 evalUndoRenderCurve(vec3 col)
{
    vec2 fMinMax;
    vec2 nMinMax;
    
    float eps = 0.00001;
    
    
    fMinMax.x = min(min(col.r,col.g),col.b);
    fMinMax.y = max(max(col.r,col.g),col.b);
    
    vec2 t = pow(fMinMax,vec2(3.14453125));
    
    nMinMax = pow(fMinMax,vec2(0.8125))*0.3828125*(1.0-t) + (1.0-pow(1.0-fMinMax,vec2(0.69140625)))*t;
    
    fMinMax.y = (nMinMax.y-nMinMax.x)/(fMinMax.y-fMinMax.x+eps);
    
    return (col-fMinMax.x)*fMinMax.y + nMinMax.x;
}

vec3 evalToneCurve(vec3 curve[8],vec3 col)
{
    float fMin,fMax;
    float nMin,nMax;
    float eps = 0.00001;
    
    fMin = min(min(col.r,col.g),col.b);
    fMax = max(max(col.r,col.g),col.b);
    
    nMin = spline8Value(curve,fMin);
    nMax = spline8Value(curve,fMax);
    
    fMax = (nMax-nMin)/(fMax-fMin+eps);
    
    return (col-fMin)*fMax + nMin;
}

//***functions
%s

void main(void)
{
    highp vec4 col = texture(uSampler, vTexCoord);
    highp vec3 rgbCol = col.rgb;

    //***rgbLinearStages
    %s

#if EditorModeEnabled
    vec2 fMinMax;
    vec2 nMinMax;
    vec2 endPin;
    
    float pinWeight;
    float eps = 0.00001;
    
    const highp mat3 rgbToProphoto = mat3(0.529285,0.098394,0.016823,
                                          0.330046,0.873493,0.117671,
                                          0.140669,0.028113,0.865506);
    
    const highp mat3 prophotoToRgb = mat3(2.034302,-0.228882,-0.00842285,
                                          -0.727295,1.23168945,-0.15332,
                                          -0.307007,-0.00280762,1.16174316);

    rgbCol = evalUndoRenderCurve( rgbToProphoto * clamp(rgbCol,0.0001,0.999) );
    
    //***stagesPreExposure
    %s

    rgbCol *= exposureGain;
    
    pinWeight = max(exposure,0.0)/8.488339269336635;
    pinWeight *= pinWeight;
    pinWeight *= pinWeight;
    fMinMax.x = min(min(rgbCol.r,rgbCol.g),rgbCol.b);
    fMinMax.y = max(max(rgbCol.r,rgbCol.g),rgbCol.b);
    
    nMinMax = fMinMax;
    
    nMinMax = min(nMinMax,1.0);
    nMinMax = pow(nMinMax,vec2(1.0/2.2));
    endPin = pow(1.0-nMinMax,vec2(exposure));
    nMinMax = 1.0-endPin;
    endPin *= endPin;
    endPin = 1.0-endPin*pinWeight;
    nMinMax *= endPin;
    
    nMinMax = pow(nMinMax,vec2(2.2));
    
    fMinMax.y = (nMinMax.y-nMinMax.x)/(fMinMax.y-fMinMax.x+eps);
    
    rgbCol = (rgbCol-fMinMax.x)*fMinMax.y + nMinMax.x;
    
    //***stagesPreTone
    %s

    rgbCol = evalToneCurve(renderCurve,rgbCol);
    
    //***stagesPostTone
    %s
    fragColor = vec4( prophotoToRgb * rgbCol, 1.0 );
#else
    fragColor = vec4( rgbCol, 1.0 );
#endif
}
